@@ -42,6 +42,7 @@ int one_shot_rlwrap = FALSE;                 /* -o option: whether to close the
 char *prompt_regexp = NULL;		     /* -O option: only ever "cook" prompts matching this regexp */
 int colour_the_prompt = FALSE;	             /* -p option: whether we should paint the prompt */
 int renice = FALSE;                          /* -R option: whether to be nicer than command */
+int mirror_arguments = FALSE;                /* -U option: whether to mirror command's arguments */
 int wait_before_prompt =  40;	             /* -w option: how long we wait before deciding we have a cookable prompt (in msec)) */
 int polling = FALSE;                         /* -W option: always give select() a small (=wait_before_prompt) timeout. */
 int impatient_prompt = TRUE;                 /* show raw prompt as soon as possible, even before we cook it. may result in "flashy" prompt */
@@ -56,7 +57,8 @@ int slave_pty_sensing_fd;		     /* slave pty (client uses this to communicate wi
 				      * client's terminal settings */
 FILE *debug_fp = NULL;  	     /* filehandle of debugging log */
 char *program_name, *command_name;   /* "rlwrap" (or whatever has been symlinked to rlwrap)  and (base-)name of command */
-char *command_line = "";             /* command plus arguments */
+char *rlwrap_command_line = "";      /* rlwrap command line (rlwrap -options command <command_args> */
+char *command_line = "";             /* command <command_args> */
 int within_line_edit = FALSE;	     /* TRUE while user is editing input */
 pid_t command_pid = 0;		     /* pid of child (client), or 0 before child is born */
 int i_am_child = FALSE;		     /* Am I child or parent? after forking, child will set this to TRUE */
@@ -96,11 +98,11 @@ static void test_main(void);
 
 /* options */
 #ifdef GETOPT_GROKS_OPTIONAL_ARGS
-static char optstring[] = "+:a::Ab:cC:d::D:e:f:F:g:hH:iIl:nNM:m::oO:p::P:q:rRs:S:t:Tvw:Wz:";
+static char optstring[] = "+:a::Ab:cC:d::D:e:f:F:g:hH:iIl:nNM:m::oO:p::P:q:rRs:S:t:TUvw:Wz:";
 /* +: is not really documented. configure checks wheteher it works as expected
    if not, GETOPT_GROKS_OPTIONAL_ARGS is undefined. @@@ */
 #else
-static char optstring[] = "+:a:Ab:cC:d:D:e:f:F:g:hH:iIl:nNM:m:oO:p:P:q:rRs:S:t:Tvw:Wz:";	
+static char optstring[] = "+:a:Ab:cC:d:D:e:f:F:g:hH:iIl:nNM:m:oO:p:P:q:rRs:S:t:TUvw:Wz:";	
 #endif
 
 #ifdef HAVE_GETOPT_LONG
@@ -136,6 +138,7 @@ static struct option longopts[] = {
   {"substitute-prompt",    	required_argument, 	NULL, 'S'},           
   {"set-terminal-name",         required_argument,      NULL, 't'},        
   {"test-terminal",  		no_argument, 		NULL, 'T'},
+  {"mirror-arguments",          no_argument, 		NULL, 'U'},
   {"version", 			no_argument, 		NULL, 'v'},
   {"wait-before-prompt",        required_argument,      NULL, 'w'},    
   {"polling",                   no_argument,            NULL, 'W'},
@@ -155,16 +158,16 @@ int
 main(int argc, char **argv)
 { 
   char *command_name;
-  char *command_line = unsplit_with(argc, argv, " ");  
-
+  rlwrap_command_line = unsplit_with(argc, argv, " ");     
   init_completer();
   command_name = read_options_and_command_name(argc, argv);
+  command_line = unsplit_with(argc - optind, argv, " ");
   
   /* by now, optind points to <command>, and &argv[optind] is <command>'s argv */
   if (!isatty(STDIN_FILENO) && execvp(argv[optind], &argv[optind]) < 0)
     /* if stdin is not a tty, just execute <command> */ 
     myerror(FATAL|USE_ERRNO, "Cannot execute %s", argv[optind]);	
-  init_rlwrap(command_line);
+  init_rlwrap(rlwrap_command_line);
   install_signal_handlers();	
   block_all_signals();
   fork_child(command_name, argv);
@@ -184,15 +187,12 @@ main(int argc, char **argv)
 static void
 fork_child(char *command_name, char **argv)
 {
-  char *arg = argv[optind], *p, **argp;
+  char *arg = argv[optind], *p;
   int pid;
 
-  command_line = mysavestring(arg);
-  for (argp = argv + optind + 1; *argp; argp++) {
-    command_line = append_and_free_old (command_line, " ");
-    command_line = append_and_free_old (command_line, *argp);
-  }
-
+  if (mirror_arguments)
+    mirror_args_init(&argv[optind]);
+  
   pid = my_pty_fork(&master_pty_fd, &saved_terminal_settings, &winsize);
   if (pid > 0)			/* parent: */
     return;
@@ -358,6 +358,8 @@ main_loop()
 	prompt_is_still_uncooked = FALSE;
       } else if (polling) {
         completely_mirror_slaves_special_characters();
+        if (mirror_arguments)
+          mirror_args(command_pid);
         continue;
       } else {
 	myerror(FATAL|NOERRNO, "unexpected select() timeout");
@@ -403,7 +405,7 @@ main_loop()
 	}
 	  
 	completely_mirror_slaves_output_settings(); /* some programs (e.g. joe) need this. Gasp!! */	
-        
+        mirror_args(command_pid);        
 	
         if (skip_rlwrap()) { /* Race condition here! The client may just have finished an emacs session and
 			        returned to cooked mode, while its ncurses-riddled output is stil waiting for us to be processed. */
@@ -814,7 +816,7 @@ read_options_and_command_name(int argc, char **argv)
 	
       break;
     case 'q': rl_basic_quote_characters = mysavestring(optarg); break;
-    case 'r':	remember_for_completion = TRUE;	break;
+    case 'r': remember_for_completion = TRUE;	break;
     case 'R': renice = TRUE;	break;
     case 's':
       histsize = atoi(optarg);
@@ -824,11 +826,12 @@ read_options_and_command_name(int argc, char **argv)
       }
       break;
     case 'S': substitute_prompt =  mysavestring(optarg);break;
-    case 't':	client_term_name=mysavestring(optarg);break;
+    case 't': client_term_name=mysavestring(optarg);break;
 #ifdef DEBUG
-    case 'T':	test_terminal(); exit(EXIT_SUCCESS);
+    case 'T': test_terminal(); exit(EXIT_SUCCESS);
 #endif
-    case 'v':	printf("rlwrap %s\n",  VERSION); exit(EXIT_SUCCESS);
+    case 'U': mirror_arguments = TRUE; break;
+    case 'v': printf("rlwrap %s\n",  VERSION); exit(EXIT_SUCCESS);
     case 'w':
       wait_before_prompt = atoi(optarg);
       if (wait_before_prompt < 0) {
