@@ -194,6 +194,7 @@ extern char *program_name, *command_name;
 extern int always_readline;
 extern int complete_filenames;
 extern pid_t command_pid;
+extern char *rlwrap_command_line;
 extern char *command_line;
 extern char *extra_char_after_completion;
 extern int i_am_child;
@@ -219,6 +220,7 @@ extern int ansi_colour_aware;
 extern int colour_the_prompt;
 extern int received_WINCH;
 extern int prompt_is_still_uncooked;
+extern int mirror_arguments;
 extern int impatient_prompt;
 extern int we_just_got_a_signal_or_EOF;
 extern int remember_for_completion;
@@ -307,7 +309,8 @@ void  mysetenv(const char *name, const char *value);
 void  set_ulimit(int resource, long value);
 void  usage(int status);
 int   open_unique_tempfile(const char *suffix, char **tmpfile_name);
-
+void  mirror_args_init(char **argv);
+void  mirror_args(pid_t command_pid);
 
 /* flags to use for the error_flags argument to myerror */
 #define FATAL     2
@@ -356,6 +359,7 @@ char *append_and_free_old(char *str1, const char *str2);
 char *mangle_char_for_debug_log(char c, int quote_me);
 char *mangle_string_for_debug_log(const char *string, int maxlen);
 char *mangle_buffer_for_debug_log(const char *buffer, int length);
+char *mem2str(const char *mem, int size);
 char *search_and_replace(char *patt, char *repl, const char *string,
                          int cursorpos, int *line, int *col);
 char *first_of(char **strings);
@@ -494,6 +498,7 @@ void filter_test(void);
 #  define DEBUG_READLINE                         4
 #  define DEBUG_MEMORY_MANAGEMENT                8   /* used with malloc_debug.c */
 #  define DEBUG_FILTERING                        16
+#  define DEBUG_COMPLETION                       32
 #  define DEBUG_AD_HOC                           64  /* only used during rlwrap development */
 #  define DEBUG_WITH_TIMESTAMPS                  128 /* add timestamps to every line in debug log    */
 #  define FORCE_HOMEGROWN_REDISPLAY              256 /* force use of my_homegrown_redisplay()        */
